@@ -1,5 +1,7 @@
+import io
 import logging
 import os
+import shutil
 import uuid
 
 import ruamel.yaml
@@ -68,7 +70,7 @@ class Entry(object):
                 data += "{name}: []\n".format(**field)
             else:
                 data += "{name}: \n".format(**field)
-        return ruamel.yaml.load(data, Loader=ruamel.yaml.RoundTripLoader)
+        return self.collection.yaml.load(data)
 
     @property
     def yaml_data(self):
@@ -81,7 +83,9 @@ class Entry(object):
                 v = to_dump.get(field['name'], '')
                 if v:
                     to_dump[field['name']] = str(v)
-        return ruamel.yaml.dump(to_dump, Dumper=ruamel.yaml.RoundTripDumper)
+        s_io = io.StringIO()
+        self.collection.yaml.dump(to_dump, s_io)
+        return s_io.getvalue()
 
     @property
     def idterm(self):
@@ -130,13 +134,13 @@ class Collection(object):
     def __init__(self, directory=None, itemdir='items'):
         self.basedir = directory or os.getcwd()
         self.itemdir = os.path.join(self.basedir, itemdir)
+        self.yaml = ruamel.yaml.YAML()
+        self.yaml.preserve_quotes = True
         try:
             with open(os.path.join(self.basedir, 'settings.yaml')) as fp:
-                self.settings = ruamel.yaml.load(
-                    fp, ruamel.yaml.RoundTripLoader
-                )
+                self.settings = self.yaml.load(fp)
         except FileNotFoundError:
-            self.settings = ruamel.yaml.safe_load("{}")
+            self.settings = self.yaml.load("{}")
         self.fields = self._load_field_types()
         os.makedirs(os.path.join(self.basedir, '.lesana'), exist_ok=True)
         if 'lang' in self.settings:
@@ -151,9 +155,6 @@ class Collection(object):
         else:
             self.stemmer = xapian.Stem('english')
         self._enquire = None
-        # This selects whether to load all other yaml files with
-        # safe_load or load + RoundTripLoader
-        self.safe = False
         self.entry_class = Entry
 
     def _get_subsubclasses(self, cls):
@@ -166,11 +167,15 @@ class Collection(object):
         for t in self._get_subsubclasses(types.LesanaType):
             type_loaders[t.name] = t
         fields = {}
-        for field in self.settings.get('fields', []):
+        for i, field in enumerate(self.settings.get('fields', [])):
             try:
                 fields[field['name']] = type_loaders[field['type']](
                     field,
                     type_loaders,
+                    # value slot 0 is used to store the filename, and we
+                    # reserve a few more slots just in case they are
+                    # needed by lesana or some derivative
+                    value_index=i + 16,
                 )
             except KeyError:
                 # unknown fields are treated as if they were
@@ -186,10 +191,7 @@ class Collection(object):
 
     def _index_file(self, fname, cache):
         with open(os.path.join(self.itemdir, fname)) as fp:
-            if self.safe:
-                data = ruamel.yaml.safe_load(fp)
-            else:
-                data = ruamel.yaml.load(fp, ruamel.yaml.RoundTripLoader)
+            data = self.yaml.load(fp)
         entry = self.entry_class(self, data, fname)
         valid, errors = entry.validate()
         if not valid:
@@ -201,39 +203,9 @@ class Collection(object):
         doc = xapian.Document()
         self.indexer.set_document(doc)
 
-        # Fields with prefix, for field search
-        for field in self.indexed_fields:
-            if field['multi']:
-                values = entry.data.get(field['name'])
-            else:
-                values = [entry.data.get(field['name'])]
-            if not values:
-                values = []
-            for v in values:
-                try:
-                    self.indexer.index_text(str(v), 1, field['prefix'])
-                except ValueError as e:
-                    logging.info(
-                        "Not indexing empty? value {}: {}".format(
-                            entry.data.get(field['name']), str(e)
-                        )
-                    )
-        # unprefixed fields, for full text search
-        for field in self.indexed_fields:
-            if field.get('free_search', False):
-                if field['multi']:
-                    values = entry.data.get(field['name'])
-                else:
-                    values = [entry.data.get(field['name'])]
-                if not values:
-                    values = []
-                for v in values:
-                    try:
-                        self.indexer.index_text(v)
-                        self.indexer.increase_termpos()
-                    except ValueError:
-                        # probably already logged earlier
-                        pass
+        for field, loader in self.fields.items():
+            loader.index(doc, self.indexer, entry.data.get(field))
+
         doc.set_data(entry.yaml_data)
         doc.add_boolean_term(entry.idterm)
         doc.add_value(0, entry.fname.encode('utf-8'))
@@ -257,7 +229,7 @@ class Collection(object):
                 )
         return fields
 
-    def update_cache(self, fnames=None):
+    def update_cache(self, fnames=None, reset=False):
         """
         Update the xapian db with the data in files.
 
@@ -265,8 +237,13 @@ class Collection(object):
 
         If no files have been passed, add everything.
 
+        if ``reset`` the existing xapian db is deleted before indexing
+
         Return the number of files that have been added to the cache.
         """
+        if reset:
+            shutil.rmtree(os.path.join(self.basedir, '.lesana'))
+        os.makedirs(os.path.join(self.basedir, '.lesana'), exist_ok=True)
         cache = xapian.WritableDatabase(
             os.path.join(self.basedir, '.lesana/xapian'),
             xapian.DB_CREATE_OR_OPEN,
@@ -341,7 +318,7 @@ class Collection(object):
             )
         return cache
 
-    def start_search(self, querystring):
+    def start_search(self, querystring, sort_by=None):
         """
         Prepare a search for querystring.
         """
@@ -358,6 +335,21 @@ class Collection(object):
         self._enquire = xapian.Enquire(cache)
         self._enquire.set_query(query)
 
+        if sort_by:
+            keymaker = xapian.MultiValueKeyMaker()
+            for k in sort_by:
+                if k.startswith('+'):
+                    reverse = False
+                    slot = self.fields[k[1:]].value_index
+                elif k.startswith('-'):
+                    reverse = True
+                    slot = self.fields[k[1:]].value_index
+                else:
+                    reverse = False
+                    slot = self.fields[k].value_index
+                keymaker.add_value(slot, reverse)
+            self._enquire.set_sort_by_key_then_relevance(keymaker, False)
+
     def get_search_results(self, offset=0, pagesize=12):
         if not self._enquire:
             return
@@ -389,12 +381,7 @@ class Collection(object):
 
     def _doc_to_entry(self, doc):
         fname = doc.get_value(0).decode('utf-8')
-        if self.safe:
-            data = ruamel.yaml.safe_load(doc.get_data())
-        else:
-            data = ruamel.yaml.load(
-                doc.get_data(), ruamel.yaml.RoundTripLoader
-            )
+        data = self.yaml.load(doc.get_data())
         entry = self.entry_class(self, data=data, fname=fname,)
         return entry
 
@@ -522,13 +509,12 @@ class Collection(object):
             skel = resource_string('lesana', 'data/settings.yaml').decode(
                 'utf-8'
             )
-            skel_dict = ruamel.yaml.load(skel, ruamel.yaml.RoundTripLoader)
+            yaml = ruamel.yaml.YAML()
+            skel_dict = yaml.load(skel)
             skel_dict['git'] = git_enabled
             skel_dict.update(settings)
             with open(filepath, 'w') as fp:
-                ruamel.yaml.dump(
-                    skel_dict, stream=fp, Dumper=ruamel.yaml.RoundTripDumper
-                )
+                yaml.dump(skel_dict, stream=fp)
         if edit_file:
             edit_file(filepath)
         if git_enabled and repo:
