@@ -213,6 +213,13 @@ class Index(Command):
             ['--collection', '-c'],
             dict(help='The collection to work on (default .)'),
         ),
+        (
+            ['--reset'],
+            dict(
+                action='store_true',
+                help='Delete the existing index and reindex from scratch.',
+            ),
+        ),
         (
             ['files'],
             dict(
@@ -229,7 +236,10 @@ class Index(Command):
             files = (os.path.basename(f) for f in self.args.files)
         else:
             files = None
-        indexed = collection.update_cache(fnames=files)
+        indexed = collection.update_cache(
+            fnames=files,
+            reset=self.args.reset
+        )
         print("Found and indexed {} entries".format(indexed))
 
 
@@ -252,6 +262,10 @@ class Search(Command):
             ['--all'],
             dict(action='store_true', help='Return all available results'),
         ),
+        (
+            ['--sort'],
+            dict(action='append', help='Sort results by a sortable field'),
+        ),
         (
             ['query'],
             dict(help='Xapian query to search in the collection', nargs='+'),
@@ -276,7 +290,10 @@ class Search(Command):
         if self.args.query == ['*']:
             results = collection.get_all_documents()
         else:
-            collection.start_search(' '.join(self.args.query))
+            collection.start_search(
+                ' '.join(self.args.query),
+                sort_by=self.args.sort
+            )
             if self.args.all:
                 results = collection.get_all_search_results()
             else:
