@@ -10,13 +10,16 @@ import logging
 
 import dateutil.parser
 
+import xapian
+
 
 class LesanaType:
     """
     Base class for lesana field types.
     """
-    def __init__(self, field, types):
+    def __init__(self, field, types, value_index=None):
         self.field = field
+        self.value_index = value_index
 
     def load(self, data):
         raise NotImplementedError
@@ -24,6 +27,48 @@ class LesanaType:
     def empty(self):
         raise NotImplementedError
 
+    def _to_index_text(self, value):
+        """
+        Prepare a value for indexing.
+        """
+        return str(value)
+
+    def _to_value(self, value):
+        """
+        Prepare a value for indexing in a value slot
+        """
+        return str(value)
+
+    def index(self, doc, indexer, value):
+        """
+        Index a value for this field type.
+
+        Override this for types that need any kind of special treatment
+        to be indexed.
+
+        See LesanaList for an idea on how to do so.
+        """
+        to_index = self.field.get('index', False)
+        if not to_index:
+            return
+        if not value:
+            logging.info(
+                "Not indexing empty value {}".format(value)
+            )
+            return
+        prefix = self.field.get('prefix', 'X' + self.field['name'].upper())
+        indexer.index_text(self._to_index_text(value), 1, prefix)
+        if to_index == 'free':
+            indexer.index_text(self._to_index_text(value))
+            indexer.increase_termpos()
+        if self.field.get('sortable', False):
+            if self.value_index and self.value_index >= 16:
+                doc.add_value(self.value_index, self._to_value(value))
+            else:
+                logging.debug(
+                    "Index values up to 8 are reserved for internal use"
+                )
+
 
 class LesanaString(LesanaType):
     """
@@ -66,6 +111,18 @@ class LesanaInt(LesanaType):
     def empty(self):
         return 0
 
+    def _to_index_text(self, value):
+        """
+        Prepare a value for indexing.
+        """
+        return str(value)
+
+    def _to_value(self, value):
+        """
+        Prepare a value for indexing in a value slot
+        """
+        return xapian.sortable_serialise(value)
+
 
 class LesanaFloat(LesanaType):
     """
@@ -145,9 +202,11 @@ class LesanaDatetime(LesanaType):
             return data
         if isinstance(data, datetime.date):
             return datetime.datetime(data.year, data.month, data.day)
+        # compatibility with dateutil before 2.8
+        ParserError = getattr(dateutil.parser, 'ParserError', ValueError)
         try:
             return dateutil.parser.parse(data)
-        except dateutil.parser.ParserError:
+        except ParserError:
             raise LesanaValueError(
                 "Invalid value for datetime field: {}".format(data)
             )
@@ -235,8 +294,8 @@ class LesanaList(LesanaType):
 
     name = 'list'
 
-    def __init__(self, field, types):
-        super().__init__(field, types)
+    def __init__(self, field, types, value_index=None):
+        super().__init__(field, types, value_index)
         try:
             self.sub_type = types[field['list']](field, types)
         except KeyError:
@@ -261,6 +320,10 @@ class LesanaList(LesanaType):
     def empty(self):
         return []
 
+    def index(self, doc, indexer, value):
+        for v in value:
+            self.sub_type.index(doc, indexer, v)
+
 
 class LesanaValueError(ValueError):
     """
