@@ -8,12 +8,13 @@ import git
 import ruamel.yaml
 
 import lesana
+from . import utils
 
 
 class testEntries(unittest.TestCase):
     def setUp(self):
         self.tmpdir = tempfile.mkdtemp()
-        shutil.copytree('tests/data/simple', self.tmpdir, dirs_exist_ok=True)
+        utils.copytree('tests/data/simple', self.tmpdir, dirs_exist_ok=True)
         self.collection = lesana.Collection(self.tmpdir)
         self.basepath = self.collection.itemdir
         self.filenames = []
@@ -120,7 +121,7 @@ class testEntries(unittest.TestCase):
 class testEmptyCollection(unittest.TestCase):
     def setUp(self):
         self.tmpdir = tempfile.mkdtemp()
-        shutil.copytree('tests/data/empty', self.tmpdir, dirs_exist_ok=True)
+        utils.copytree('tests/data/empty', self.tmpdir, dirs_exist_ok=True)
         self.collection = lesana.Collection(self.tmpdir)
 
     def tearDown(self):
@@ -136,7 +137,7 @@ class testEmptyCollection(unittest.TestCase):
 class testSimpleCollection(unittest.TestCase):
     def setUp(self):
         self.tmpdir = tempfile.mkdtemp()
-        shutil.copytree('tests/data/simple', self.tmpdir, dirs_exist_ok=True)
+        utils.copytree('tests/data/simple', self.tmpdir, dirs_exist_ok=True)
         self.collection = lesana.Collection(self.tmpdir)
 
     def tearDown(self):
@@ -153,12 +154,6 @@ class testSimpleCollection(unittest.TestCase):
         self.collection.update_cache()
         self.assertIsNotNone(self.collection.stemmer)
 
-    def test_load_safe(self):
-        # Simply run the code with self.collection.safe = True to check
-        # that it doesn't break.
-        self.collection.safe = True
-        self.collection.update_cache()
-
     def test_full_search(self):
         self.collection.start_search('Item')
         res = self.collection.get_all_search_results()
@@ -225,6 +220,9 @@ class testSimpleCollection(unittest.TestCase):
         self.assertEqual(len(cm.output), 1)
         self.assertIn("non_existing_file", cm.output[0])
 
+    def test_index_reset(self):
+        self.collection.update_cache(reset=True)
+
     def test_get_entry_missing_eid(self):
         entry = self.collection.entry_from_eid('this is not an eid')
         self.assertIsNone(entry)
@@ -264,7 +262,7 @@ class testSimpleCollection(unittest.TestCase):
 class testComplexCollection(unittest.TestCase):
     def setUp(self):
         self.tmpdir = tempfile.mkdtemp()
-        shutil.copytree('tests/data/complex', self.tmpdir, dirs_exist_ok=True)
+        utils.copytree('tests/data/complex', self.tmpdir, dirs_exist_ok=True)
         self.collection = lesana.Collection(self.tmpdir)
 
     def tearDown(self):
@@ -278,7 +276,7 @@ class testComplexCollection(unittest.TestCase):
         )
         self.assertEqual(len(self.collection.settings['fields']), 9)
         self.assertIsNotNone(self.collection.stemmer)
-        self.assertEqual(len(self.collection.indexed_fields), 6)
+        self.assertEqual(len(self.collection.indexed_fields), 7)
 
     def test_index(self):
         self.collection.update_cache()
@@ -300,7 +298,6 @@ class testComplexCollection(unittest.TestCase):
 
     def test_empty_data(self):
         entry = lesana.Entry(self.collection)
-        print(entry.yaml_data)
         self.assertIn("name: ''", entry.yaml_data)
         self.assertIn('with_default', entry.yaml_data)
         self.assertIn('amount: 0', entry.yaml_data)
@@ -321,11 +318,49 @@ class testComplexCollection(unittest.TestCase):
         for f in to_test:
             self.assertIsInstance(self.collection.fields[f[0]], f[1])
 
+    def test_comments_are_preserved(self):
+        e = self.collection.entry_from_eid('5084bc6e94f24dc6976629282ef30419')
+        yaml_data = e.yaml_data
+        self.assertTrue(
+            yaml_data.startswith("# This entry has a comment at the beginning")
+        )
+        self.assertTrue(
+            yaml_data.endswith("# and a comment at the end\n")
+        )
+
+    def test_data_is_stored_as_written_on_file(self):
+        e = self.collection.entry_from_eid('5084bc6e94f24dc6976629282ef30419')
+        fname = 'tests/data/complex/items/' + \
+            '5084bc6e94f24dc6976629282ef30419.yaml'
+        with open(fname, 'r') as fp:
+            self.assertEqual(e.yaml_data, fp.read())
+
+    def test_sorted_search(self):
+        # search in ascending order
+        self.collection.start_search('Amount', sort_by=['amount'])
+        res = self.collection.get_search_results()
+        matches = list(res)
+        self.assertEqual(len(matches), 4)
+        self.assertEqual(matches[0].data['amount'], 2)
+        self.assertEqual(matches[1].data['amount'], 10)
+        self.assertEqual(matches[2].data['amount'], 15)
+        self.assertEqual(matches[3].data['amount'], 20)
+
+        # and in descending order
+        self.collection.start_search('Amount', sort_by=['-amount'])
+        res = self.collection.get_search_results()
+        matches = list(res)
+        self.assertEqual(len(matches), 4)
+        self.assertEqual(matches[0].data['amount'], 20)
+        self.assertEqual(matches[1].data['amount'], 15)
+        self.assertEqual(matches[2].data['amount'], 10)
+        self.assertEqual(matches[3].data['amount'], 2)
+
 
 class testCollectionWithErrors(unittest.TestCase):
     def setUp(self):
         self.tmpdir = tempfile.mkdtemp()
-        shutil.copytree('tests/data/wrong', self.tmpdir, dirs_exist_ok=True)
+        utils.copytree('tests/data/wrong', self.tmpdir, dirs_exist_ok=True)
         self.collection = lesana.Collection(self.tmpdir)
 
     def tearDown(self):
@@ -445,7 +480,7 @@ class testCollectionCreation(unittest.TestCase):
 
     def test_deletion(self):
         shutil.copy('tests/data/simple/settings.yaml', self.tmpdir)
-        shutil.copytree(
+        utils.copytree(
             'tests/data/simple/items', os.path.join(self.tmpdir, 'items'),
         )
         collection = lesana.Collection.init(self.tmpdir)
@@ -471,7 +506,7 @@ class testCollectionCreation(unittest.TestCase):
 
     def test_partial_eid_deletion(self):
         shutil.copy('tests/data/simple/settings.yaml', self.tmpdir)
-        shutil.copytree(
+        utils.copytree(
             'tests/data/simple/items', os.path.join(self.tmpdir, 'items'),
         )
         collection = lesana.Collection.init(self.tmpdir)
@@ -505,7 +540,7 @@ class testCollectionCreation(unittest.TestCase):
 
     def test_git_adding(self):
         shutil.copy('tests/data/simple/settings.yaml', self.tmpdir)
-        shutil.copytree(
+        utils.copytree(
             'tests/data/simple/items', os.path.join(self.tmpdir, 'items'),
         )
         collection = lesana.Collection.init(self.tmpdir)
